@@ -124,6 +124,7 @@ class BlackboxTestCase(TestCase):
 def self_test_suite() -> unittest.TestSuite:
     names = [
         "annotate",
+        "approxidate",
         "archive",
         "attrs",
         "bisect",
@@ -183,7 +184,9 @@ def self_test_suite() -> unittest.TestSuite:
         "server",
         "sparse_patterns",
         "stash",
+        "stripspace",
         "submodule",
+        "trailers",
         "utils",
         "walk",
         "web",
