@@ -3,7 +3,7 @@
 #
 # SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
-# General Public License as public by the Free Software Foundation; version 2.0
+# General Public License as published by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
 # modify it under the terms of either of these two licenses.
 #
@@ -820,12 +820,10 @@ class StackedConfigTests(TestCase):
         install_dir = os.path.join("C:", "foo", "Git")
         self.overrideEnv("PATH", os.path.join(install_dir, "cmd"))
         with patch("os.path.exists", return_value=True):
-            paths = set(get_win_system_paths())
+            paths = list(get_win_system_paths())
+        # Should only return the current Git config path, not legacy paths
         self.assertEqual(
-            {
-                os.path.join(os.environ.get("PROGRAMDATA"), "Git", "config"),
-                os.path.join(install_dir, "etc", "gitconfig"),
-            },
+            [os.path.join(install_dir, "etc", "gitconfig")],
             paths,
         )
 
@@ -842,15 +840,135 @@ class StackedConfigTests(TestCase):
                 "winreg.QueryValueEx",
                 return_value=(install_dir, winreg.REG_SZ),
             ):
-                paths = set(get_win_system_paths())
+                paths = list(get_win_system_paths())
+        # Should only return the current Git config path, not legacy paths
         self.assertEqual(
-            {
-                os.path.join(os.environ.get("PROGRAMDATA"), "Git", "config"),
-                os.path.join(install_dir, "etc", "gitconfig"),
-            },
+            [os.path.join(install_dir, "etc", "gitconfig")],
             paths,
         )
 
+    @skipIf(sys.platform != "win32", "Windows specific config location.")
+    def test_windows_legacy_paths(self) -> None:
+        from dulwich.config import get_win_legacy_system_paths
+
+        install_dir = os.path.join("C:", "foo", "Git")
+        self.overrideEnv("PATH", os.path.join(install_dir, "cmd"))
+        self.overrideEnv("PROGRAMDATA", "C:\\ProgramData")
+        with patch("os.path.exists", return_value=True):
+            paths = list(get_win_legacy_system_paths())
+        # Should return all paths including legacy PROGRAMDATA location
+        self.assertIn(
+            os.path.join("C:\\ProgramData", "Git", "config"),
+            paths,
+        )
+        self.assertIn(
+            os.path.join(install_dir, "etc", "gitconfig"),
+            paths,
+        )
+
+    def test_git_config_global(self) -> None:
+        with tempfile.NamedTemporaryFile(
+            suffix=".gitconfig", delete=False
+        ) as global_config:
+            # Write test config
+            global_config.write(b"[user]\n\tname = Test User\n")
+            global_config.flush()
+            config_path = global_config.name
+
+        try:
+            self.overrideEnv("GIT_CONFIG_GLOBAL", config_path)
+            backends = StackedConfig.default_backends()
+
+            # Should have at least one backend (the global config)
+            self.assertGreater(len(backends), 0)
+
+            # Check that the custom global config is loaded
+            paths = [backend.path for backend in backends if hasattr(backend, "path")]
+            self.assertIn(
+                config_path.encode() if isinstance(paths[0], bytes) else config_path,
+                paths,
+            )
+        finally:
+            os.unlink(config_path)
+
+    def test_git_config_system(self) -> None:
+        with tempfile.NamedTemporaryFile(
+            suffix=".gitconfig", delete=False
+        ) as system_config:
+            # Write test config
+            system_config.write(b"[core]\n\tautocrlf = true\n")
+            system_config.flush()
+            config_path = system_config.name
+
+        try:
+            self.overrideEnv("GIT_CONFIG_SYSTEM", config_path)
+            self.overrideEnv("GIT_CONFIG_NOSYSTEM", None)  # Ensure it's not set
+            backends = StackedConfig.default_backends()
+
+            # Should have at least one backend (the system config)
+            self.assertGreater(len(backends), 0)
+
+            # Check that the custom system config is loaded
+            paths = [backend.path for backend in backends if hasattr(backend, "path")]
+            self.assertIn(
+                config_path.encode() if isinstance(paths[0], bytes) else config_path,
+                paths,
+            )
+        finally:
+            os.unlink(config_path)
+
+    def test_debug_logging(self) -> None:
+        """Test that debug logging outputs gitconfig paths."""
+        import logging
+
+        # Set up a custom handler to capture log messages
+        log_messages = []
+        handler = logging.Handler()
+        handler.emit = lambda record: log_messages.append(record.getMessage())
+
+        # Get the dulwich.config logger and add our handler
+        logger = logging.getLogger("dulwich.config")
+        old_level = logger.level
+        logger.setLevel(logging.DEBUG)
+        logger.addHandler(handler)
+
+        try:
+            # Call default_backends which now logs unconditionally
+            StackedConfig.default_backends()
+
+            # Check that debug messages were logged
+            self.assertTrue(
+                any("Loading gitconfig from paths:" in msg for msg in log_messages),
+                "Debug logging should log gitconfig paths",
+            )
+        finally:
+            # Clean up
+            logger.removeHandler(handler)
+            logger.setLevel(old_level)
+
+    def test_git_config_nosystem_precedence(self) -> None:
+        # GIT_CONFIG_SYSTEM should take precedence over GIT_CONFIG_NOSYSTEM
+        with tempfile.NamedTemporaryFile(
+            suffix=".gitconfig", delete=False
+        ) as system_config:
+            system_config.write(b"[core]\n\ttest = value\n")
+            system_config.flush()
+            config_path = system_config.name
+
+        try:
+            self.overrideEnv("GIT_CONFIG_SYSTEM", config_path)
+            self.overrideEnv("GIT_CONFIG_NOSYSTEM", "1")  # This should be ignored
+            backends = StackedConfig.default_backends()
+
+            # Check that the custom system config is still loaded
+            paths = [backend.path for backend in backends if hasattr(backend, "path")]
+            self.assertIn(
+                config_path.encode() if isinstance(paths[0], bytes) else config_path,
+                paths,
+            )
+        finally:
+            os.unlink(config_path)
+
 
 class EscapeValueTests(TestCase):
     def test_nothing(self) -> None:
