@@ -24,6 +24,8 @@
 from dulwich.line_ending import (
     BlobNormalizer,
     LineEndingFilter,
+    TreeBlobNormalizer,
+    check_safecrlf,
     convert_crlf_to_lf,
     convert_lf_to_crlf,
     get_clean_filter_autocrlf,
@@ -365,6 +367,96 @@ class BlobNormalizerTests(TestCase):
         self.assertIs(result, blob)
 
 
+class TreeBlobNormalizerTests(TestCase):
+    """Test the TreeBlobNormalizer class for existing file handling."""
+
+    def setUp(self) -> None:
+        super().setUp()
+        from dulwich.config import ConfigDict
+        from dulwich.object_store import MemoryObjectStore
+
+        self.config = ConfigDict()
+        self.gitattributes = {}
+        self.object_store = MemoryObjectStore()
+
+    def test_autocrlf_input_existing_files(self) -> None:
+        """Test that autocrlf=input normalizes existing files with CRLF."""
+        # Create a tree with an existing file
+        from dulwich.objects import Tree
+
+        tree = Tree()
+        tree[b"existing.txt"] = (0o100644, b"a" * 40)  # dummy sha
+        self.object_store.add_object(tree)
+
+        # Create normalizer with autocrlf=input
+        normalizer = TreeBlobNormalizer(
+            self.config,
+            self.gitattributes,
+            self.object_store,
+            tree.id,
+            autocrlf=b"input",
+        )
+
+        # Create blob with CRLF line endings
+        blob = Blob()
+        blob.data = b"line1\r\nline2\r\n"
+
+        # Should convert CRLF to LF on checkin even for existing files
+        result = normalizer.checkin_normalize(blob, b"existing.txt")
+        self.assertEqual(result.data, b"line1\nline2\n")
+
+    def test_autocrlf_false_existing_files(self) -> None:
+        """Test that autocrlf=false does not normalize existing files."""
+        # Create a tree with an existing file
+        from dulwich.objects import Tree
+
+        tree = Tree()
+        tree[b"existing.txt"] = (0o100644, b"a" * 40)  # dummy sha
+        self.object_store.add_object(tree)
+
+        # Create normalizer with autocrlf=false
+        normalizer = TreeBlobNormalizer(
+            self.config,
+            self.gitattributes,
+            self.object_store,
+            tree.id,
+            autocrlf=b"false",
+        )
+
+        # Create blob with CRLF line endings
+        blob = Blob()
+        blob.data = b"line1\r\nline2\r\n"
+
+        # Should NOT convert for existing files when autocrlf=false
+        result = normalizer.checkin_normalize(blob, b"existing.txt")
+        self.assertIs(result, blob)
+
+    def test_autocrlf_input_new_files(self) -> None:
+        """Test that autocrlf=input normalizes new files."""
+        # Create empty tree (no existing files)
+        from dulwich.objects import Tree
+
+        tree = Tree()
+        self.object_store.add_object(tree)
+
+        # Create normalizer with autocrlf=input
+        normalizer = TreeBlobNormalizer(
+            self.config,
+            self.gitattributes,
+            self.object_store,
+            tree.id,
+            autocrlf=b"input",
+        )
+
+        # Create blob with CRLF line endings
+        blob = Blob()
+        blob.data = b"line1\r\nline2\r\n"
+
+        # Should convert CRLF to LF for new files
+        result = normalizer.checkin_normalize(blob, b"new.txt")
+        self.assertEqual(result.data, b"line1\nline2\n")
+
+
 class LineEndingIntegrationTests(TestCase):
     """Integration tests for line ending conversion with the filter system."""
 
@@ -394,9 +486,12 @@ class LineEndingIntegrationTests(TestCase):
         gitattributes = GitAttributes(patterns)
 
         # Create normalizer
-        from dulwich.filters import FilterBlobNormalizer
+        from dulwich.filters import FilterBlobNormalizer, FilterContext
 
-        normalizer = FilterBlobNormalizer(self.config, gitattributes, self.registry)
+        filter_context = FilterContext(self.registry)
+        normalizer = FilterBlobNormalizer(
+            self.config, gitattributes, filter_context=filter_context
+        )
 
         # Test round trip
         blob = Blob()
@@ -428,6 +523,12 @@ class LineEndingIntegrationTests(TestCase):
             def smudge(self, data):
                 return b"LFS content"
 
+            def cleanup(self):
+                pass
+
+            def reuse(self, config, filter_name):
+                return False
+
         self.registry.register_driver("lfs", MockLFSFilter())
 
         # Different files use different filters
@@ -439,9 +540,12 @@ class LineEndingIntegrationTests(TestCase):
         ]
         gitattributes = GitAttributes(patterns)
 
-        from dulwich.filters import FilterBlobNormalizer
+        from dulwich.filters import FilterBlobNormalizer, FilterContext
 
-        normalizer = FilterBlobNormalizer(self.config, gitattributes, self.registry)
+        filter_context = FilterContext(self.registry)
+        normalizer = FilterBlobNormalizer(
+            self.config, gitattributes, filter_context=filter_context
+        )
 
         # Text file gets line ending conversion
         text_blob = Blob()
@@ -454,3 +558,133 @@ class LineEndingIntegrationTests(TestCase):
         bin_blob.data = b"binary content"
         result = normalizer.checkin_normalize(bin_blob, b"test.bin")
         self.assertEqual(result.data, b"LFS pointer")
+
+
+class LineEndingFilterFromConfigTests(TestCase):
+    """Test LineEndingFilter.from_config classmethod."""
+
+    def test_from_config_none(self) -> None:
+        """Test from_config with no config."""
+        # No config, not for text attr - no conversion
+        filter = LineEndingFilter.from_config(None, for_text_attr=False)
+        self.assertIsNone(filter.clean_conversion)
+        self.assertIsNone(filter.smudge_conversion)
+        self.assertEqual(filter.safecrlf, b"false")
+
+        # No config, for text attr - normalize on checkin
+        filter = LineEndingFilter.from_config(None, for_text_attr=True)
+        self.assertIsNotNone(filter.clean_conversion)
+        self.assertIsNone(filter.smudge_conversion)
+        self.assertEqual(filter.safecrlf, b"false")
+
+    def test_from_config_autocrlf_true(self) -> None:
+        """Test from_config with autocrlf=true."""
+        from dulwich.config import ConfigDict
+
+        config = ConfigDict()
+        config.set(b"core", b"autocrlf", b"true")
+
+        filter = LineEndingFilter.from_config(config, for_text_attr=False)
+        self.assertIsNotNone(filter.clean_conversion)
+        self.assertIsNotNone(filter.smudge_conversion)
+        self.assertEqual(filter.safecrlf, b"false")
+
+    def test_from_config_with_safecrlf(self) -> None:
+        """Test from_config with safecrlf setting."""
+        from dulwich.config import ConfigDict
+
+        config = ConfigDict()
+        config.set(b"core", b"autocrlf", b"input")
+        config.set(b"core", b"safecrlf", b"warn")
+
+        filter = LineEndingFilter.from_config(config, for_text_attr=False)
+        self.assertIsNotNone(filter.clean_conversion)
+        self.assertIsNone(filter.smudge_conversion)
+        self.assertEqual(filter.safecrlf, b"warn")
+
+    def test_from_config_text_attr_overrides(self) -> None:
+        """Test that for_text_attr=True always normalizes on checkin."""
+        from dulwich.config import ConfigDict
+
+        config = ConfigDict()
+        config.set(b"core", b"autocrlf", b"false")
+
+        # Even with autocrlf=false, text attr should normalize
+        filter = LineEndingFilter.from_config(config, for_text_attr=True)
+        self.assertIsNotNone(filter.clean_conversion)
+        # Smudge should still be None since autocrlf=false
+        self.assertIsNone(filter.smudge_conversion)
+
+
+class SafeCRLFTests(TestCase):
+    """Test core.safecrlf functionality."""
+
+    def test_safecrlf_false(self) -> None:
+        """Test that safecrlf=false allows any conversion."""
+        original = b"line1\r\nline2\r\n"
+        converted = b"line1\nline2\n"
+        # Should not raise
+        check_safecrlf(original, converted, b"false", b"test.txt")
+
+    def test_safecrlf_true_safe_conversion(self) -> None:
+        """Test that safecrlf=true allows safe conversions."""
+        # CRLF -> LF -> CRLF is reversible
+        original = b"line1\r\nline2\r\n"
+        converted = b"line1\nline2\n"
+        # Should not raise because conversion is reversible
+        check_safecrlf(original, converted, b"true", b"test.txt")
+
+    def test_safecrlf_true_unsafe_conversion(self) -> None:
+        """Test that safecrlf=true fails on unsafe conversions."""
+        # Mixed line endings would be lost
+        original = b"line1\r\nline2\nline3\r\n"
+        converted = b"line1\nline2\nline3\n"
+        # Should raise because converting back gives all CRLF
+        with self.assertRaises(ValueError) as cm:
+            check_safecrlf(original, converted, b"true", b"test.txt")
+        self.assertIn("CRLF would be replaced by LF", str(cm.exception))
+
+    def test_safecrlf_warn(self) -> None:
+        """Test that safecrlf=warn issues warnings."""
+        # Mixed line endings would be lost
+        original = b"line1\r\nline2\nline3\r\n"
+        converted = b"line1\nline2\nline3\n"
+        # Should warn but not raise
+        with self.assertLogs("dulwich.line_ending", level="WARNING") as cm:
+            check_safecrlf(original, converted, b"warn", b"test.txt")
+            self.assertEqual(len(cm.output), 1)
+            self.assertIn("CRLF would be replaced by LF", cm.output[0])
+
+    def test_lineending_filter_with_safecrlf(self) -> None:
+        """Test LineEndingFilter with safecrlf enabled."""
+        # Test with safecrlf=true
+        filter_strict = LineEndingFilter(
+            clean_conversion=convert_crlf_to_lf,
+            smudge_conversion=None,
+            binary_detection=False,
+            safecrlf=b"true",
+        )
+
+        # Safe conversion should work
+        safe_data = b"line1\r\nline2\r\n"
+        result = filter_strict.clean(safe_data, b"test.txt")
+        self.assertEqual(result, b"line1\nline2\n")
+
+        # Unsafe conversion should fail
+        unsafe_data = b"line1\r\nline2\nline3\r\n"
+        with self.assertRaises(ValueError):
+            filter_strict.clean(unsafe_data, b"test.txt")
+
+        # Test with safecrlf=warn
+        filter_warn = LineEndingFilter(
+            clean_conversion=convert_crlf_to_lf,
+            smudge_conversion=None,
+            binary_detection=False,
+            safecrlf=b"warn",
+        )
+
+        # Should warn but still convert
+        with self.assertLogs("dulwich.line_ending", level="WARNING") as cm:
+            result = filter_warn.clean(unsafe_data, b"test.txt")
+            self.assertEqual(result, b"line1\nline2\nline3\n")
+            self.assertEqual(len(cm.output), 1)
