@@ -38,6 +38,7 @@ from dulwich.patch import (
     write_object_diff,
     write_tree_diff,
 )
+from dulwich.tests.utils import make_commit
 
 from . import DependencyMissing, SkipTest, TestCase
 
@@ -45,12 +46,16 @@ from . import DependencyMissing, SkipTest, TestCase
 class WriteCommitPatchTests(TestCase):
     def test_simple_bytesio(self) -> None:
         f = BytesIO()
-        c = Commit()
-        c.committer = c.author = b"Jelmer <jelmer@samba.org>"
-        c.commit_time = c.author_time = 1271350201
-        c.commit_timezone = c.author_timezone = 0
-        c.message = b"This is the first line\nAnd this is the second line.\n"
-        c.tree = Tree().id
+        c = make_commit(
+            author=b"Jelmer <jelmer@samba.org>",
+            committer=b"Jelmer <jelmer@samba.org>",
+            author_time=1271350201,
+            commit_time=1271350201,
+            author_timezone=0,
+            commit_timezone=0,
+            message=b"This is the first line\nAnd this is the second line.\n",
+            tree=Tree().id,
+        )
         write_commit_patch(f, c, b"CONTENTS", (1, 1), version="custom")
         f.seek(0)
         lines = f.readlines()
@@ -632,12 +637,16 @@ class DiffTests(TestCase):
 
 class GetSummaryTests(TestCase):
     def test_simple(self) -> None:
-        c = Commit()
-        c.committer = c.author = b"Jelmer <jelmer@samba.org>"
-        c.commit_time = c.author_time = 1271350201
-        c.commit_timezone = c.author_timezone = 0
-        c.message = b"This is the first line\nAnd this is the second line.\n"
-        c.tree = Tree().id
+        c = make_commit(
+            author=b"Jelmer <jelmer@samba.org>",
+            committer=b"Jelmer <jelmer@samba.org>",
+            author_time=1271350201,
+            commit_time=1271350201,
+            author_timezone=0,
+            commit_timezone=0,
+            message=b"This is the first line\nAnd this is the second line.\n",
+            tree=Tree().id,
+        )
         self.assertEqual("This-is-the-first-line", get_summary(c))
 
 
